@@ -83,6 +83,16 @@ def resample_patient(data, seg, original_spacing, target_spacing, order_data=3,
             do_separate_z = False
             axis = None
 
+    if axis is not None:
+        if len(axis) == 3:
+            # every axis has the spacing
+            axis = (0, )
+        elif len(axis) == 2:
+            print("WARNING: axis has len 2, axis: %s, spacing: %s, target_spacing: %s" % (str(axis), original_spacing, target_spacing))
+            do_separate_z = False
+        else:
+            pass
+
     if data is not None:
         data_reshaped = resample_data_or_seg(data, new_shape, False, axis, order_data, do_separate_z, cval=cval_data,
                                              order_z=order_z_data)
@@ -302,9 +312,8 @@ class GenericPreprocessor(object):
                                                             force_separate_z=force_separate_z)
         return data.astype(np.float32), seg, properties
 
-    def _run_star(self, args):
-        target_spacing, case_identifier, output_folder_stage, cropped_output_dir, force_separate_z = args
-
+    def _run_internal(self, target_spacing, case_identifier, output_folder_stage, cropped_output_dir, force_separate_z,
+                      all_classes):
         data, seg, properties = self.load_cropped(cropped_output_dir, case_identifier)
 
         data = data.transpose((0, *[i + 1 for i in self.transpose_forward]))
@@ -315,6 +324,24 @@ class GenericPreprocessor(object):
 
         all_data = np.vstack((data, seg)).astype(np.float32)
 
+        # we need to find out where the classes are and sample some random locations
+        # let's do 10.000 samples per class
+        # seed this for reproducibility!
+        num_samples = 10000
+        min_percent_coverage = 0.01 # at least 1% of the class voxels need to be selected, otherwise it may be too sparse
+        rndst = np.random.RandomState(1234)
+        class_locs = {}
+        for c in all_classes:
+            all_locs = np.argwhere(all_data[-1] == c)
+            if len(all_locs) == 0:
+                class_locs[c] = []
+                continue
+            target_num_samples = min(num_samples, len(all_locs))
+            target_num_samples = max(target_num_samples, len(all_locs) * min_percent_coverage)
+            selected = all_locs[rndst.choice(len(all_locs), target_num_samples, replace=False)]
+            class_locs[c] = selected
+            print(c, target_num_samples)
+        properties['class_locations'] = class_locs
 
         print("saving: ", os.path.join(output_folder_stage, "%s.npz" % case_identifier))
         np.savez_compressed(os.path.join(output_folder_stage, "%s.npz" % case_identifier),
@@ -344,6 +371,10 @@ class GenericPreprocessor(object):
 
         assert len(num_threads) == num_stages
 
+        # we need to know which classes are present in this dataset so that we can precompute where these classes are
+        # located. This is needed for oversampling foreground
+        all_classes = load_pickle(join(input_folder_with_cropped_npz, 'dataset_properties.pkl'))['all_classes']
+
         for i in range(num_stages):
             all_args = []
             output_folder_stage = os.path.join(output_folder, data_identifier + "_stage%d" % i)
@@ -351,10 +382,10 @@ class GenericPreprocessor(object):
             spacing = target_spacings[i]
             for j, case in enumerate(list_of_cropped_npz_files):
                 case_identifier = get_case_identifier_from_npz(case)
-                args = spacing, case_identifier, output_folder_stage, input_folder_with_cropped_npz, force_separate_z
+                args = spacing, case_identifier, output_folder_stage, input_folder_with_cropped_npz, force_separate_z, all_classes
                 all_args.append(args)
             p = Pool(num_threads[i])
-            p.map(self._run_star, all_args)
+            p.starmap(self._run_internal, all_args)
             p.close()
             p.join()
 
@@ -441,6 +472,100 @@ class Preprocessor3DDifferentResampling(GenericPreprocessor):
         return data, seg, properties
 
 
+class Preprocessor3DBetterResampling(GenericPreprocessor):
+    """
+    This preprocessor always uses force_separate_z=False. It does resampling to the target spacing with third
+    order spline for data (just like GenericPreprocessor) and seg (unlike GenericPreprocessor). It never does separate
+    resampling in z.
+    """
+    def resample_and_normalize(self, data, target_spacing, properties, seg=None, force_separate_z=False):
+        """
+        data and seg must already have been transposed by transpose_forward. properties are the un-transposed values
+        (spacing etc)
+        :param data:
+        :param target_spacing:
+        :param properties:
+        :param seg:
+        :param force_separate_z:
+        :return:
+        """
+        if force_separate_z is not False:
+            print("WARNING: Preprocessor3DBetterResampling always uses force_separate_z=False. "
+                  "You specified %s. Your choice is overwritten" % str(force_separate_z))
+            force_separate_z = False
+
+        # be safe
+        assert force_separate_z is False
+
+        # target_spacing is already transposed, properties["original_spacing"] is not so we need to transpose it!
+        # data, seg are already transposed. Double check this using the properties
+        original_spacing_transposed = np.array(properties["original_spacing"])[self.transpose_forward]
+        before = {
+            'spacing': properties["original_spacing"],
+            'spacing_transposed': original_spacing_transposed,
+            'data.shape (data is transposed)': data.shape
+        }
+
+        # remove nans
+        data[np.isnan(data)] = 0
+
+        data, seg = resample_patient(data, seg, np.array(original_spacing_transposed), target_spacing, 3, 3,
+                                     force_separate_z=force_separate_z, order_z_data=99999, order_z_seg=99999,
+                                     separate_z_anisotropy_threshold=self.resample_separate_z_anisotropy_threshold)
+        after = {
+            'spacing': target_spacing,
+            'data.shape (data is resampled)': data.shape
+        }
+        print("before:", before, "\nafter: ", after, "\n")
+
+        if seg is not None:  # hippocampus 243 has one voxel with -2 as label. wtf?
+            seg[seg < -1] = 0
+
+        properties["size_after_resampling"] = data[0].shape
+        properties["spacing_after_resampling"] = target_spacing
+        use_nonzero_mask = self.use_nonzero_mask
+
+        assert len(self.normalization_scheme_per_modality) == len(data), "self.normalization_scheme_per_modality " \
+                                                                         "must have as many entries as data has " \
+                                                                         "modalities"
+        assert len(self.use_nonzero_mask) == len(data), "self.use_nonzero_mask must have as many entries as data" \
+                                                        " has modalities"
+
+        for c in range(len(data)):
+            scheme = self.normalization_scheme_per_modality[c]
+            if scheme == "CT":
+                # clip to lb and ub from train data foreground and use foreground mn and sd from training data
+                assert self.intensityproperties is not None, "ERROR: if there is a CT then we need intensity properties"
+                mean_intensity = self.intensityproperties[c]['mean']
+                std_intensity = self.intensityproperties[c]['sd']
+                lower_bound = self.intensityproperties[c]['percentile_00_5']
+                upper_bound = self.intensityproperties[c]['percentile_99_5']
+                data[c] = np.clip(data[c], lower_bound, upper_bound)
+                data[c] = (data[c] - mean_intensity) / std_intensity
+                if use_nonzero_mask[c]:
+                    data[c][seg[-1] < 0] = 0
+            elif scheme == "CT2":
+                # clip to lb and ub from train data foreground, use mn and sd form each case for normalization
+                assert self.intensityproperties is not None, "ERROR: if there is a CT then we need intensity properties"
+                lower_bound = self.intensityproperties[c]['percentile_00_5']
+                upper_bound = self.intensityproperties[c]['percentile_99_5']
+                mask = (data[c] > lower_bound) & (data[c] < upper_bound)
+                data[c] = np.clip(data[c], lower_bound, upper_bound)
+                mn = data[c][mask].mean()
+                sd = data[c][mask].std()
+                data[c] = (data[c] - mn) / sd
+                if use_nonzero_mask[c]:
+                    data[c][seg[-1] < 0] = 0
+            else:
+                if use_nonzero_mask[c]:
+                    mask = seg[-1] >= 0
+                else:
+                    mask = np.ones(seg.shape[1:], dtype=bool)
+                data[c][mask] = (data[c][mask] - data[c][mask].mean()) / (data[c][mask].std() + 1e-8)
+                data[c][mask == 0] = 0
+        return data, seg, properties
+
+
 class PreprocessorFor2D(GenericPreprocessor):
     def __init__(self, normalization_scheme_per_modality, use_nonzero_mask, transpose_forward: (tuple, list), intensityproperties=None):
         super(PreprocessorFor2D, self).__init__(normalization_scheme_per_modality, use_nonzero_mask,
@@ -456,16 +581,21 @@ class PreprocessorFor2D(GenericPreprocessor):
         maybe_mkdir_p(output_folder)
         all_args = []
         num_stages = len(target_spacings)
+
+        # we need to know which classes are present in this dataset so that we can precompute where these classes are
+        # located. This is needed for oversampling foreground
+        all_classes = load_pickle(join(input_folder_with_cropped_npz, 'dataset_properties.pkl'))['all_classes']
+
         for i in range(num_stages):
             output_folder_stage = os.path.join(output_folder, data_identifier + "_stage%d" % i)
             maybe_mkdir_p(output_folder_stage)
             spacing = target_spacings[i]
             for j, case in enumerate(list_of_cropped_npz_files):
                 case_identifier = get_case_identifier_from_npz(case)
-                args = spacing, case_identifier, output_folder_stage, input_folder_with_cropped_npz, force_separate_z
+                args = spacing, case_identifier, output_folder_stage, input_folder_with_cropped_npz, force_separate_z, all_classes
                 all_args.append(args)
         p = Pool(num_threads)
-        p.map(self._run_star, all_args)
+        p.starmap(self._run_internal, all_args)
         p.close()
         p.join()
 
@@ -535,3 +665,36 @@ class PreprocessorFor2D(GenericPreprocessor):
                 data[c][mask == 0] = 0
         print("normalization done")
         return data, seg, properties
+
+
+class PreprocessorFor2D_noNormalization(GenericPreprocessor):
+    def resample_and_normalize(self, data, target_spacing, properties, seg=None, force_separate_z=None):
+        original_spacing_transposed = np.array(properties["original_spacing"])[self.transpose_forward]
+        before = {
+            'spacing': properties["original_spacing"],
+            'spacing_transposed': original_spacing_transposed,
+            'data.shape (data is transposed)': data.shape
+        }
+        target_spacing[0] = original_spacing_transposed[0]
+        data, seg = resample_patient(data, seg, np.array(original_spacing_transposed), target_spacing, 3, 1,
+                                     force_separate_z=force_separate_z, order_z_data=0, order_z_seg=0,
+                                     separate_z_anisotropy_threshold=self.resample_separate_z_anisotropy_threshold)
+        after = {
+            'spacing': target_spacing,
+            'data.shape (data is resampled)': data.shape
+        }
+        print("before:", before, "\nafter: ", after, "\n")
+
+        if seg is not None:  # hippocampus 243 has one voxel with -2 as label. wtf?
+            seg[seg < -1] = 0
+
+        properties["size_after_resampling"] = data[0].shape
+        properties["spacing_after_resampling"] = target_spacing
+        use_nonzero_mask = self.use_nonzero_mask
+
+        assert len(self.normalization_scheme_per_modality) == len(data), "self.normalization_scheme_per_modality " \
+                                                                         "must have as many entries as data has " \
+                                                                         "modalities"
+        assert len(self.use_nonzero_mask) == len(data), "self.use_nonzero_mask must have as many entries as data" \
+                                                        " has modalities"
+        return data, seg, properties
\ No newline at end of file
