@@ -92,22 +92,27 @@ def load_dataset(folder):
     dataset = OrderedDict()
     for c in case_identifiers:
         dataset[c] = OrderedDict()
-        dataset[c]['data_file'] = join(folder, "%s.npz"%c)
-        with open(join(folder, "%s.pkl"%c), 'rb') as f:
-            dataset[c]['properties'] = pickle.load(f)
+        dataset[c]['data_file'] = join(folder, "%s.npz" % c)
+
+        # dataset[c]['properties'] = load_pickle(join(folder, "%s.pkl" % c))
+        dataset[c]['properties_file'] = join(folder, "%s.pkl" % c)
+
         if dataset[c].get('seg_from_prev_stage_file') is not None:
-            dataset[c]['seg_from_prev_stage_file'] = join(folder, "%s_segs.npz"%c)
+            dataset[c]['seg_from_prev_stage_file'] = join(folder, "%s_segs.npz" % c)
     return dataset
 
 
-def crop_2D_image_force_fg(img, crop_size, force_class=None):
+def crop_2D_image_force_fg(img, crop_size, valid_voxels):
     """
     img must be [c, x, y]
     img[-1] must be the segmentation with segmentation>0 being foreground
     :param img:
     :param crop_size:
+    :param valid_voxels: voxels belonging to the selected class
     :return:
     """
+    assert len(valid_voxels.shape) == 2
+
     if type(crop_size) not in (tuple, list):
         crop_size = [crop_size] * (len(img.shape) - 1)
     else:
@@ -120,21 +125,11 @@ def crop_2D_image_force_fg(img, crop_size, force_class=None):
     lb_y = crop_size[1] // 2
     ub_y = img.shape[2] - crop_size[1] // 2 - crop_size[1] % 2
 
-    foreground_classes = np.unique(img[-1])
-    foreground_classes = foreground_classes[foreground_classes > 0]
-    if len(foreground_classes) == 0 or (0 in foreground_classes.shape):
-        foreground_classes = [0]
-
-    if force_class is None or force_class not in foreground_classes:
-        chosen_class = np.random.choice(foreground_classes)
-    else:
-        chosen_class = force_class
-    foreground_voxels = np.array(np.where(img[-1] == chosen_class))
-    if np.any(np.array(foreground_voxels.shape) == 0):
+    if len(valid_voxels) == 0:
         selected_center_voxel = (np.random.random_integers(lb_x, ub_x),
                                  np.random.random_integers(lb_y, ub_y))
     else:
-        selected_center_voxel = foreground_voxels[:, np.random.choice(foreground_voxels.shape[1])]
+        selected_center_voxel = valid_voxels[np.random.choice(valid_voxels.shape[1]), :]
 
     selected_center_voxel = np.array(selected_center_voxel)
     for i in range(2):
@@ -193,14 +188,31 @@ class DataLoader3D(SlimDataLoaderBase):
         self.memmap_mode = memmap_mode
         self.num_channels = None
         self.pad_sides = pad_sides
+        self.data_shape, self.seg_shape = self.determine_shapes()
 
     def get_do_oversample(self, batch_idx):
         return not batch_idx < round(self.batch_size * (1 - self.oversample_foreground_percent))
 
+    def determine_shapes(self):
+        if self.has_prev_stage:
+            num_seg = 2
+        else:
+            num_seg = 1
+
+        k = list(self._data.keys())[0]
+        if isfile(self._data[k]['data_file'][:-4] + ".npy"):
+            case_all_data = np.load(self._data[k]['data_file'][:-4] + ".npy", self.memmap_mode)
+        else:
+            case_all_data = np.load(self._data[k]['data_file'])['data']
+        num_color_channels = case_all_data.shape[0] - num_seg
+        data_shape = (self.batch_size, num_color_channels, *self.patch_size)
+        seg_shape = (self.batch_size, num_seg, *self.patch_size)
+        return data_shape, seg_shape
+
     def generate_train_batch(self):
         selected_keys = np.random.choice(self.list_of_keys, self.batch_size, True, None)
-        data = []
-        seg = []
+        data = np.zeros(self.data_shape, dtype=np.float32)
+        seg = np.zeros(self.seg_shape, dtype=np.float32)
         case_properties = []
         for j, i in enumerate(selected_keys):
             # oversampling foreground will improve stability of model training, especially if many patches are empty
@@ -210,7 +222,8 @@ class DataLoader3D(SlimDataLoaderBase):
             else:
                 force_fg = False
 
-            case_properties.append(self._data[i]['properties'])
+            properties = load_pickle(self._data[i]['properties_file'])
+            case_properties.append(properties)
             # cases are stores as npz, but we require unpack_dataset to be run. This will decompress them into npy
             # which is much faster to access
             if isfile(self._data[i]['data_file'][:-4] + ".npy"):
@@ -269,24 +282,23 @@ class DataLoader3D(SlimDataLoaderBase):
                 bbox_z_lb = np.random.randint(lb_z, ub_z + 1)
             else:
                 # this saves us a np.unique. Preprocessing already did that for all cases. Neat.
-                foreground_classes = np.array(self._data[i]['properties']['classes'])
+                foreground_classes = np.array([i for i in properties['class_locations'].keys() if len(properties['class_locations'][i]) != 0])
                 foreground_classes = foreground_classes[foreground_classes > 0]
+
                 if len(foreground_classes) == 0:
-                    selected_class = 0
+                    # this only happens if some image does not contain foreground voxels at all
+                    selected_class = None
+                    voxels_of_that_class = None
+                    print('case does not contain any foreground classes', i)
                 else:
                     selected_class = np.random.choice(foreground_classes)
 
-                # we could precompute that to save CPU time but I am too lazy right now
-                voxels_of_that_class = np.argwhere(case_all_data[-1] == selected_class)
+                    # these values should have been precomputed
+                    if 'class_locations' not in properties.keys():
+                        raise RuntimeError("Please rerun the preprocessing with the newest version of nnU-Net!")
+                    voxels_of_that_class = properties['class_locations'][selected_class]
 
-                # voxels_of_that_class should always contain some voxels we chose selected_class from the classes that
-                # are actually present in the case. There is however one slight chance that foreground_classes includes
-                # a class that is not anymore present in the preprocessed data. That is because np.unique is called on
-                # the data in their original resolution but the data/segmentations here are present in some resampled
-                # resolution. In extremely rare cases (and if the ground truth was bad = single annoteted pixels) then
-                # a class can get lost in the preprocessing. This is not a problem at all because that segmentation was
-                # most likely faulty anyways but it may lead to crashes here.
-                if len(voxels_of_that_class) != 0:
+                if voxels_of_that_class is not None:
                     selected_voxel = voxels_of_that_class[np.random.choice(len(voxels_of_that_class))]
                     # selected voxel is center voxel. Subtract half the patch size to get lower bbox voxel.
                     # Make sure it is within the bounds of lb and ub
@@ -294,8 +306,7 @@ class DataLoader3D(SlimDataLoaderBase):
                     bbox_y_lb = max(lb_y, selected_voxel[1] - self.patch_size[1] // 2)
                     bbox_z_lb = max(lb_z, selected_voxel[2] - self.patch_size[2] // 2)
                 else:
-                    # If the selected class is indeed not present then we fall back to random cropping. We can do that
-                    # because this case is extremely rare.
+                    # If the image does not contain any foreground classes, we fall back to random cropping
                     bbox_x_lb = np.random.randint(lb_x, ub_x + 1)
                     bbox_y_lb = np.random.randint(lb_y, ub_y + 1)
                     bbox_z_lb = np.random.randint(lb_z, ub_z + 1)
@@ -319,7 +330,6 @@ class DataLoader3D(SlimDataLoaderBase):
             # Why not just concatenate them here and forget about the if statements? Well that's because segneeds to
             # be padded with -1 constant whereas seg_from_previous_stage needs to be padded with 0s (we could also
             # remove label -1 in the data augmentation but this way it is less error prone)
-
             case_all_data = case_all_data[:, valid_bbox_x_lb:valid_bbox_x_ub,
                                              valid_bbox_y_lb:valid_bbox_y_ub,
                                              valid_bbox_z_lb:valid_bbox_z_ub]
@@ -347,20 +357,15 @@ class DataLoader3D(SlimDataLoaderBase):
                                                  'constant', **{'constant_values': 0})
                 case_all_data_segonly = np.concatenate((case_all_data_segonly, seg_from_previous_stage), 0)
 
-            data.append(case_all_data_donly[None])
-            seg.append(case_all_data_segonly[None])
-
-        # this is bad. Slow. Better preallocate data and set with np.zeros(). But this way we don't have to know how
-        # many color channels data has and how many seg channels there are
-        data = np.vstack(data)
-        seg = np.vstack(seg)
+            data[j] = case_all_data_donly
+            seg[j] = case_all_data_segonly
 
         return {'data':data, 'seg':seg, 'properties':case_properties, 'keys': selected_keys}
 
 
 class DataLoader2D(SlimDataLoaderBase):
-    def __init__(self, data, patch_size, final_patch_size, batch_size, transpose=None,
-                 oversample_foreground_percent=0.0, memmap_mode="r", pseudo_3d_slices=1, pad_mode="edge",
+    def __init__(self, data, patch_size, final_patch_size, batch_size, oversample_foreground_percent=0.0,
+                 memmap_mode="r", pseudo_3d_slices=1, pad_mode="edge",
                  pad_kwargs_data=None, pad_sides=None):
         """
         This is the basic data loader for 2D networks. It uses preprocessed data as produced by my (Fabian) preprocessing.
@@ -394,9 +399,6 @@ class DataLoader2D(SlimDataLoaderBase):
         self.pseudo_3d_slices = pseudo_3d_slices
         self.oversample_foreground_percent = oversample_foreground_percent
         self.final_patch_size = final_patch_size
-        if transpose is not None:
-            assert isinstance(transpose, (list, tuple)), "Transpose must be either None or be a tuple/list representing the new axis order (3 ints)"
-        self.transpose = transpose
         self.patch_size = patch_size
         self.list_of_keys = list(self._data.keys())
         self.need_to_pad = np.array(patch_size) - np.array(final_patch_size)
@@ -406,30 +408,44 @@ class DataLoader2D(SlimDataLoaderBase):
                 pad_sides = np.array(pad_sides)
             self.need_to_pad += pad_sides
         self.pad_sides = pad_sides
+        self.data_shape, self.seg_shape = self.determine_shapes()
+        if len(self._data.keys()) < 2000:
+            # this uses more ram because we now have the class_locations precomputed. Only do this if the number
+            # of training cases is 'small'
+            self.load_all_properties_files()
 
-    @property
-    def all_possible_slices(self):
-        try:
-            return self._all_possible_slices
-        except AttributeError:
-            slices = []
-            for key in sorted(self.list_of_keys):
-                shape = self._data[key]["properties"]["size_after_resampling"]
-                for s in range(shape[0]):
-                    slices.append((key, s))
-            self._all_possible_slices = slices
-            return slices
+    def determine_shapes(self):
+        num_seg = 1
+
+        k = list(self._data.keys())[0]
+        if isfile(self._data[k]['data_file'][:-4] + ".npy"):
+            case_all_data = np.load(self._data[k]['data_file'][:-4] + ".npy", self.memmap_mode)
+        else:
+            case_all_data = np.load(self._data[k]['data_file'])['data']
+        num_color_channels = case_all_data.shape[0] - num_seg
+        data_shape = (self.batch_size, num_color_channels, *self.patch_size)
+        seg_shape = (self.batch_size, num_seg, *self.patch_size)
+        return data_shape, seg_shape
 
     def get_do_oversample(self, batch_idx):
         return not batch_idx < round(self.batch_size * (1 - self.oversample_foreground_percent))
 
+    def load_all_properties_files(self):
+        for i in self._data.keys():
+            self._data[i]['properties'] = load_pickle(self._data[i]['properties_file'])
+
     def generate_train_batch(self):
         selected_keys = np.random.choice(self.list_of_keys, self.batch_size, True, None)
-        data = []
-        seg = []
+
+        data = np.zeros(self.data_shape, dtype=np.float32)
+        seg = np.zeros(self.seg_shape, dtype=np.float32)
+
         case_properties = []
         for j, i in enumerate(selected_keys):
-            properties = self._data[i]['properties']
+            if 'properties' in self._data[i].keys():
+                properties = self._data[i]['properties']
+            else:
+                properties = load_pickle(self._data[i]['properties_file'])
             case_properties.append(properties)
 
             if self.get_do_oversample(j):
@@ -447,42 +463,35 @@ class DataLoader2D(SlimDataLoaderBase):
             if len(case_all_data.shape) == 3:
                 case_all_data = case_all_data[:, None]
 
-            if self.transpose is not None:
-                leading_axis = self.transpose[0]
-            else:
-                leading_axis = 0
-
+            # first select a slice. This can be either random (no force fg) or guaranteed to contain some class
             if not force_fg:
-                random_slice = np.random.choice(case_all_data.shape[leading_axis + 1])
+                random_slice = np.random.choice(case_all_data.shape[1])
+                selected_class = None
             else:
-                # select one class, then select a slice that contains that class
-                classes_in_slice_per_axis = properties.get("classes_in_slice_per_axis")
-                possible_classes = np.unique(properties['classes'])
-                possible_classes = possible_classes[possible_classes > 0]
-                if len(possible_classes) > 0 and not (0 in possible_classes.shape):
-                    selected_class = np.random.choice(possible_classes)
+                foreground_classes = np.array([i for i in properties['class_locations'].keys() if len(properties['class_locations'][i]) != 0])
+                foreground_classes = foreground_classes[foreground_classes > 0]
+                if len(foreground_classes) == 0:
+                    selected_class = None
+                    random_slice = np.random.choice(case_all_data.shape[1])
+                    print('case does not contain any foreground classes', i)
                 else:
-                    selected_class = 0
-                if classes_in_slice_per_axis is not None:
-                    valid_slices = classes_in_slice_per_axis[leading_axis][selected_class]
-                else:
-                    valid_slices = np.where(np.sum(case_all_data[-1] == selected_class, axis=[i for i in range(3) if i != leading_axis]))[0]
-                if len(valid_slices) != 0:
-                    random_slice = np.random.choice(valid_slices)
-                else:
-                    random_slice = np.random.choice(case_all_data.shape[leading_axis + 1])
+                    selected_class = np.random.choice(foreground_classes)
+                    # these values should have been precomputed
+                    if 'class_locations' not in properties.keys():
+                        raise RuntimeError("Please rerun the preprocessing with the newest version of nnU-Net!")
 
+                    voxels_of_that_class = properties['class_locations'][selected_class]
+                    valid_slices = np.unique(voxels_of_that_class[:, 0])
+                    random_slice = np.random.choice(valid_slices)
+                    voxels_of_that_class = voxels_of_that_class[voxels_of_that_class[:, 0] == random_slice]
+
+            # now crop case_all_data to contain just the slice of interest. If we want additional slice above and
+            # below the current slice, here is where we get them. We stack those as additional color channels
             if self.pseudo_3d_slices == 1:
-                if leading_axis == 0:
-                    case_all_data = case_all_data[:, random_slice]
-                elif leading_axis == 1:
-                    case_all_data = case_all_data[:, :, random_slice]
-                else:
-                    case_all_data = case_all_data[:, :, :, random_slice]
-                if self.transpose is not None and self.transpose[1] > self.transpose[2]:
-                    case_all_data = case_all_data.transpose(0, 2, 1)
+                case_all_data = case_all_data[:, random_slice]
             else:
-                assert leading_axis == 0, "pseudo_3d_slices works only without transpose for now!"
+                # this is very deprecated and will probably not work anymore. If you intend to use this you need to
+                # check this!
                 mn = random_slice - (self.pseudo_3d_slices - 1) // 2
                 mx = random_slice + (self.pseudo_3d_slices - 1) // 2 + 1
                 valid_mn = max(mn, 0)
@@ -504,39 +513,73 @@ class DataLoader2D(SlimDataLoaderBase):
                 case_all_data = case_all_data.reshape((-1, case_all_data.shape[-2], case_all_data.shape[-1]))
                 case_all_data = np.concatenate((case_all_data, case_all_seg), 0)
 
-            num_seg = 1
+            # case all data should now be (c, x, y)
+            assert len(case_all_data.shape) == 3
 
-            # why we need this is a little complicated. It has to do with downstream random cropping during data
-            # augmentation. Basically we will rotate the patch and then to a center crop of size self.final_patch_size.
-            # Depending on the rotation, scaling and elastic deformation parameters, self.patch_size has to be large
-            # enough to prevent border artifacts from being present in the final patch
-            new_shp = None
-            if np.any(self.need_to_pad) > 0:
-                new_shp = np.array(case_all_data.shape[1:] + self.need_to_pad)
-                if np.any(new_shp - np.array(self.patch_size) < 0):
-                    new_shp = np.max(np.vstack((new_shp[None], np.array(self.patch_size)[None])), 0)
-            else:
-                if np.any(np.array(case_all_data.shape[1:]) - np.array(self.patch_size) < 0):
-                    new_shp = np.max(
-                        np.vstack((np.array(case_all_data.shape[1:])[None], np.array(self.patch_size)[None])), 0)
-            if new_shp is not None:
-                case_all_data_donly = pad_nd_image(case_all_data[:-num_seg], new_shp, self.pad_mode, kwargs=self.pad_kwargs_data)
-                case_all_data_segnonly = pad_nd_image(case_all_data[-num_seg:], new_shp, 'constant', kwargs={'constant_values':-1})
-                case_all_data = np.vstack((case_all_data_donly, case_all_data_segnonly))[None]
-            else:
-                case_all_data = case_all_data[None]
+            # we can now choose the bbox from -need_to_pad // 2 to shape - patch_size + need_to_pad // 2. Here we
+            # define what the upper and lower bound can be to then sample form them with np.random.randint
 
-            if not force_fg:
-                case_all_data = random_crop_2D_image_batched(case_all_data, tuple(self.patch_size))
+            need_to_pad = self.need_to_pad
+            for d in range(2):
+                # if case_all_data.shape + need_to_pad is still < patch size we need to pad more! We pad on both sides
+                # always
+                if need_to_pad[d] + case_all_data.shape[d+1] < self.patch_size[d]:
+                    need_to_pad[d] = self.patch_size[d] - case_all_data.shape[d+1]
+
+            shape = case_all_data.shape[1:]
+            lb_x = - need_to_pad[0] // 2
+            ub_x = shape[0] + need_to_pad[0] // 2 + need_to_pad[0] % 2 - self.patch_size[0]
+            lb_y = - need_to_pad[1] // 2
+            ub_y = shape[1] + need_to_pad[1] // 2 + need_to_pad[1] % 2 - self.patch_size[1]
+
+            # if not force_fg then we can just sample the bbox randomly from lb and ub. Else we need to make sure we get
+            # at least one of the foreground classes in the patch
+            if not force_fg or selected_class is None:
+                bbox_x_lb = np.random.randint(lb_x, ub_x + 1)
+                bbox_y_lb = np.random.randint(lb_y, ub_y + 1)
             else:
-                case_all_data = crop_2D_image_force_fg(case_all_data[0], tuple(self.patch_size), selected_class)[None]
-            data.append(case_all_data[:, :-num_seg])
-            seg.append(case_all_data[:, -num_seg:])
-        data = np.vstack(data)
-        seg = np.vstack(seg)
+                # this saves us a np.unique. Preprocessing already did that for all cases. Neat.
+                selected_voxel = voxels_of_that_class[np.random.choice(len(voxels_of_that_class))]
+                # selected voxel is center voxel. Subtract half the patch size to get lower bbox voxel.
+                # Make sure it is within the bounds of lb and ub
+                bbox_x_lb = max(lb_x, selected_voxel[0] - self.patch_size[0] // 2)
+                bbox_y_lb = max(lb_y, selected_voxel[1] - self.patch_size[1] // 2)
+
+            bbox_x_ub = bbox_x_lb + self.patch_size[0]
+            bbox_y_ub = bbox_y_lb + self.patch_size[1]
+
+            # whoever wrote this knew what he was doing (hint: it was me). We first crop the data to the region of the
+            # bbox that actually lies within the data. This will result in a smaller array which is then faster to pad.
+            # valid_bbox is just the coord that lied within the data cube. It will be padded to match the patch size
+            # later
+            valid_bbox_x_lb = max(0, bbox_x_lb)
+            valid_bbox_x_ub = min(shape[0], bbox_x_ub)
+            valid_bbox_y_lb = max(0, bbox_y_lb)
+            valid_bbox_y_ub = min(shape[1], bbox_y_ub)
+
+            # At this point you might ask yourself why we would treat seg differently from seg_from_previous_stage.
+            # Why not just concatenate them here and forget about the if statements? Well that's because segneeds to
+            # be padded with -1 constant whereas seg_from_previous_stage needs to be padded with 0s (we could also
+            # remove label -1 in the data augmentation but this way it is less error prone)
+
+            case_all_data = case_all_data[:, valid_bbox_x_lb:valid_bbox_x_ub,
+                            valid_bbox_y_lb:valid_bbox_y_ub]
+
+            case_all_data_donly = np.pad(case_all_data[:-1], ((0, 0),
+                                                              (-min(0, bbox_x_lb), max(bbox_x_ub - shape[0], 0)),
+                                                              (-min(0, bbox_y_lb), max(bbox_y_ub - shape[1], 0))),
+                                         self.pad_mode, **self.pad_kwargs_data)
+
+            case_all_data_segonly = np.pad(case_all_data[-1:], ((0, 0),
+                                                                (-min(0, bbox_x_lb), max(bbox_x_ub - shape[0], 0)),
+                                                                (-min(0, bbox_y_lb), max(bbox_y_ub - shape[1], 0))),
+                                           'constant', **{'constant_values': -1})
+
+            data[j] = case_all_data_donly
+            seg[j] = case_all_data_segonly
+
         keys = selected_keys
-        return {'data':data, 'seg':seg, 'properties':case_properties, "keys": keys}
-
+        return {'data': data, 'seg': seg, 'properties': case_properties, "keys": keys}
 
 
 if __name__ == "__main__":
