@@ -20,6 +20,7 @@ from nnunet.utilities.nd_softmax import softmax_helper
 from nnunet.utilities.tensor_utilities import sum_tensor
 from torch import nn
 import numpy as np
+import torch.nn.functional as F
 
 
 class GDL(nn.Module):
@@ -182,7 +183,7 @@ class SoftDiceLoss(nn.Module):
         nominator = 2 * tp + self.smooth
         denominator = 2 * tp + fp + fn + self.smooth
 
-        dc = nominator / denominator
+        dc = nominator / (denominator + 1e-8)
 
         if not self.do_bg:
             if self.batch_dice:
@@ -302,7 +303,8 @@ class SoftDiceLossSquared(nn.Module):
 
 
 class DC_and_CE_loss(nn.Module):
-    def __init__(self, soft_dice_kwargs, ce_kwargs, aggregate="sum", square_dice=False, weight_ce=1, weight_dice=1):
+    def __init__(self, soft_dice_kwargs, ce_kwargs, aggregate="sum", square_dice=False, weight_ce=1, weight_dice=1,
+                 log_dice=False, ignore_label=None):
         """
         CAREFUL. Weights for CE and Dice do not need to sum to one. You can set whatever you want.
         :param soft_dice_kwargs:
@@ -313,18 +315,46 @@ class DC_and_CE_loss(nn.Module):
         :param weight_dice:
         """
         super(DC_and_CE_loss, self).__init__()
+        if ignore_label is not None:
+            assert not square_dice, 'not implemented'
+            ce_kwargs['reduction'] = 'none'
+        self.log_dice = log_dice
         self.weight_dice = weight_dice
         self.weight_ce = weight_ce
         self.aggregate = aggregate
-        self.ce = CrossentropyND(**ce_kwargs)
+        self.ce = nn.CrossEntropyLoss(**ce_kwargs)
+
+        self.ignore_label = ignore_label
+
         if not square_dice:
             self.dc = SoftDiceLoss(apply_nonlin=softmax_helper, **soft_dice_kwargs)
         else:
             self.dc = SoftDiceLossSquared(apply_nonlin=softmax_helper, **soft_dice_kwargs)
 
     def forward(self, net_output, target):
-        dc_loss = self.dc(net_output, target) if self.weight_dice != 0 else 0
-        ce_loss = self.ce(net_output, target) if self.weight_ce != 0 else 0
+        """
+        target must be b, c, x, y(, z) with c=1
+        :param net_output:
+        :param target:
+        :return:
+        """
+        if self.ignore_label is not None:
+            assert target.shape[1] == 1, 'not implemented for one hot encoding'
+            mask = target != self.ignore_label
+            target[~mask] = 0
+            mask = mask.float()
+        else:
+            mask = None
+
+        dc_loss = self.dc(net_output, target, loss_mask=mask) if self.weight_dice != 0 else 0
+        if self.log_dice:
+            dc_loss = -torch.log(-dc_loss)
+
+        ce_loss = self.ce(net_output, target[:, 0].long()) if self.weight_ce != 0 else 0
+        if self.ignore_label is not None:
+            ce_loss *= mask[:, 0]
+            ce_loss = ce_loss.sum() / mask.sum()
+
         if self.aggregate == "sum":
             result = self.weight_ce * ce_loss + self.weight_dice * dc_loss
         else:
@@ -332,6 +362,34 @@ class DC_and_CE_loss(nn.Module):
         return result
 
 
+class DC_and_BCE_loss(nn.Module):
+    def __init__(self, bce_kwargs, soft_dice_kwargs, aggregate="sum"):
+        """
+        DO NOT APPLY NONLINEARITY IN YOUR NETWORK!
+
+        THIS LOSS IS INTENDED TO BE USED FOR BRATS REGIONS ONLY
+        :param soft_dice_kwargs:
+        :param bce_kwargs:
+        :param aggregate:
+        """
+        super(DC_and_BCE_loss, self).__init__()
+
+        self.aggregate = aggregate
+        self.ce = nn.BCEWithLogitsLoss(**bce_kwargs)
+        self.dc = SoftDiceLoss(apply_nonlin=torch.sigmoid, **soft_dice_kwargs)
+
+    def forward(self, net_output, target):
+        ce_loss = self.ce(net_output, target)
+        dc_loss = self.dc(net_output, target)
+
+        if self.aggregate == "sum":
+            result = ce_loss + dc_loss
+        else:
+            raise NotImplementedError("nah son") # reserved for other stuff (later)
+
+        return result
+
+
 class GDL_and_CE_loss(nn.Module):
     def __init__(self, gdl_dice_kwargs, ce_kwargs, aggregate="sum"):
         super(GDL_and_CE_loss, self).__init__()
