@@ -339,8 +339,7 @@ class NetworkTrainer(object):
         # match. Use heuristic to make it match
         for k, value in saved_model['state_dict'].items():
             key = k
-            if key not in curr_state_dict_keys:
-                print("duh")
+            if key not in curr_state_dict_keys and key.startswith('module.'):
                 key = key[7:]
             new_state_dict[key] = value
 
@@ -411,6 +410,7 @@ class NetworkTrainer(object):
 
         self._maybe_init_amp()
 
+        maybe_mkdir_p(self.output_folder)        
         self.plot_network_architecture()
 
         if cudnn.benchmark and cudnn.deterministic:
@@ -418,8 +418,6 @@ class NetworkTrainer(object):
                  "But torch.backends.cudnn.benchmark is True as well and this will prevent deterministic training! "
                  "If you want deterministic then set benchmark=False")
 
-        maybe_mkdir_p(self.output_folder)
-
         if not self.was_initialized:
             self.initialize(True)
 
@@ -468,11 +466,12 @@ class NetworkTrainer(object):
                     self.all_val_losses_tr_mode.append(np.mean(val_losses))
                     self.print_to_log_file("validation loss (train=True): %.4f" % self.all_val_losses_tr_mode[-1])
 
-            epoch_end_time = time()
-
             self.update_train_loss_MA()  # needed for lr scheduler and stopping of training
 
             continue_training = self.on_epoch_end()
+
+            epoch_end_time = time()
+
             if not continue_training:
                 # allows for early stopping
                 break
