@@ -29,6 +29,7 @@ from nnunet.training.data_augmentation.default_data_augmentation import default_
 from nnunet.training.dataloading.dataset_loading import unpack_dataset
 from nnunet.training.network_training.nnUNetTrainer import nnUNetTrainer
 from nnunet.utilities.nd_softmax import softmax_helper
+from sklearn.model_selection import KFold
 from torch import nn
 from torch.nn.utils import clip_grad_norm_
 from nnunet.training.learning_rate.poly_lr import poly_lr
@@ -85,7 +86,7 @@ class nnUNetTrainerV2(nnUNetTrainer):
             weights = np.array([1 / (2 ** i) for i in range(net_numpool)])
 
             # we don't use the lowest 2 outputs. Normalize weights so that they sum to 1
-            mask = np.array([True if i < net_numpool - 1 else False for i in range(net_numpool)])
+            mask = np.array([True] + [True if i < net_numpool - 1 else False for i in range(1, net_numpool)])
             weights[~mask] = 0
             weights = weights / weights.sum()
             self.ds_loss_weights = weights
@@ -185,16 +186,15 @@ class nnUNetTrainerV2(nnUNetTrainer):
     def validate(self, do_mirroring: bool = True, use_sliding_window: bool = True,
                  step_size: float = 0.5, save_softmax: bool = True, use_gaussian: bool = True, overwrite: bool = True,
                  validation_folder_name: str = 'validation_raw', debug: bool = False, all_in_gpu: bool = False,
-                 force_separate_z: bool = None, interpolation_order: int = 3, interpolation_order_z=0):
+                 segmentation_export_kwargs: dict = None):
         """
         We need to wrap this because we need to enforce self.network.do_ds = False for prediction
         """
         ds = self.network.do_ds
         self.network.do_ds = False
         ret = super().validate(do_mirroring, use_sliding_window, step_size, save_softmax, use_gaussian,
-                               overwrite, validation_folder_name, debug, all_in_gpu,
-                               force_separate_z=force_separate_z, interpolation_order=interpolation_order,
-                               interpolation_order_z=interpolation_order_z)
+                               overwrite, validation_folder_name, debug, all_in_gpu, segmentation_export_kwargs)
+
         self.network.do_ds = ds
         return ret
 
@@ -265,21 +265,50 @@ class nnUNetTrainerV2(nnUNetTrainer):
         Folds > 4 will be independent from each other
         :return:
         """
-        if self.fold == 'all' or self.fold < 5:
-            return super().do_split()
+        if self.fold == "all":
+            # if fold==all then we use all images for training and validation
+            tr_keys = val_keys = list(self.dataset.keys())
         else:
-            rnd = np.random.RandomState(seed=12345 + self.fold)
-            keys = np.sort(list(self.dataset.keys()))
-            idx_tr = rnd.choice(len(keys), int(len(keys) * 0.8), replace=False)
-            idx_val = [i for i in range(len(keys)) if i not in idx_tr]
+            splits_file = join(self.dataset_directory, "splits_final.pkl")
 
-            self.dataset_tr = OrderedDict()
-            for i in idx_tr:
-                self.dataset_tr[keys[i]] = self.dataset[keys[i]]
+            # if the split file does not exist we need to create it
+            if not isfile(splits_file):
+                self.print_to_log_file("Creating new split...")
+                splits = []
+                all_keys_sorted = np.sort(list(self.dataset.keys()))
+                kfold = KFold(n_splits=5, shuffle=True, random_state=12345)
+                for i, (train_idx, test_idx) in enumerate(kfold.split(all_keys_sorted)):
+                    train_keys = np.array(all_keys_sorted)[train_idx]
+                    test_keys = np.array(all_keys_sorted)[test_idx]
+                    splits.append(OrderedDict())
+                    splits[-1]['train'] = train_keys
+                    splits[-1]['val'] = test_keys
+                save_pickle(splits, splits_file)
 
-            self.dataset_val = OrderedDict()
-            for i in idx_val:
-                self.dataset_val[keys[i]] = self.dataset[keys[i]]
+            splits = load_pickle(splits_file)
+
+            if self.fold < len(splits):
+                tr_keys = splits[self.fold]['train']
+                val_keys = splits[self.fold]['val']
+            else:
+                self.print_to_log_file("INFO: Requested fold %d but split file only has %d folds. I am now creating a "
+                                       "random 80:20 split!" % (self.fold, len(splits)))
+                # if we request a fold that is not in the split file, create a random 80:20 split
+                rnd = np.random.RandomState(seed=12345 + self.fold)
+                keys = np.sort(list(self.dataset.keys()))
+                idx_tr = rnd.choice(len(keys), int(len(keys) * 0.8), replace=False)
+                idx_val = [i for i in range(len(keys)) if i not in idx_tr]
+                tr_keys = [keys[i] for i in idx_tr]
+                val_keys = [keys[i] for i in idx_val]
+
+        tr_keys.sort()
+        val_keys.sort()
+        self.dataset_tr = OrderedDict()
+        for i in tr_keys:
+            self.dataset_tr[i] = self.dataset[i]
+        self.dataset_val = OrderedDict()
+        for i in val_keys:
+            self.dataset_val[i] = self.dataset[i]
 
     def setup_DA_params(self):
         """
