@@ -28,8 +28,6 @@ except ImportError:
     DDP = None
 
 from batchgenerators.utilities.file_and_folder_operations import maybe_mkdir_p, join, subfiles, isfile
-from nnunet.network_architecture.generic_UNet import Generic_UNet
-from nnunet.network_architecture.initialization import InitWeights_He
 from nnunet.network_architecture.neural_network import SegmentationNetwork
 from nnunet.training.data_augmentation.default_data_augmentation import get_moreDA_augmentation
 from nnunet.training.dataloading.dataset_loading import unpack_dataset
@@ -70,7 +68,7 @@ class nnUNetTrainerV2_DDP(nnUNetTrainerV2):
         self.val_loss_MA = None
 
         self.loss = None
-        self.ce_loss = CrossentropyND()
+        self.ce_loss = nn.CrossEntropyLoss()
 
         self.global_batch_size = None  # we need to know this to properly steer oversample
 
@@ -130,38 +128,6 @@ class nnUNetTrainerV2_DDP(nnUNetTrainerV2):
         if self.local_rank == 0:
             super().print_to_log_file(*args, also_print_to_console=also_print_to_console)
 
-    def initialize_network(self):
-        """
-        This is specific to the U-Net and must be adapted for other network architectures
-        :return:
-        """
-        self.print_to_log_file(self.net_num_pool_op_kernel_sizes)
-        self.print_to_log_file(self.net_conv_kernel_sizes)
-
-        if self.threeD:
-            conv_op = nn.Conv3d
-            dropout_op = nn.Dropout3d
-            norm_op = nn.InstanceNorm3d
-
-        else:
-            conv_op = nn.Conv2d
-            dropout_op = nn.Dropout2d
-            norm_op = nn.InstanceNorm2d
-
-        norm_op_kwargs = {'eps': 1e-5, 'affine': True}
-        dropout_op_kwargs = {'p': 0, 'inplace': True}
-        net_nonlin = nn.LeakyReLU
-        net_nonlin_kwargs = {'negative_slope': 1e-2, 'inplace': True}
-        self.network = Generic_UNet(self.num_input_channels, self.base_num_features, self.num_classes,
-                                    len(self.net_num_pool_op_kernel_sizes),
-                                    self.conv_per_stage, 2, conv_op, norm_op, norm_op_kwargs, dropout_op,
-                                    dropout_op_kwargs,
-                                    net_nonlin, net_nonlin_kwargs, True, False, lambda x: x, InitWeights_He(1e-2),
-                                    self.net_num_pool_op_kernel_sizes, self.net_conv_kernel_sizes, False, True, True)
-        if torch.cuda.is_available():
-            self.network.cuda()
-        self.network.inference_apply_nonlin = softmax_helper
-
     def process_plans(self, plans):
         super().process_plans(plans)
         self.set_batch_size_and_oversample()
@@ -295,7 +261,7 @@ class nnUNetTrainerV2_DDP(nnUNetTrainerV2):
             else:
                 pass
 
-            ce_loss = self.ce_loss(output[i], target[i])
+            ce_loss = self.ce_loss(output[i], target[i][:, 0].long())
 
             # we smooth by 1e-5 to penalize false positives if tp is 0
             dice_loss = (- (nominator + 1e-5) / (denominator + 1e-5)).mean()
@@ -374,10 +340,10 @@ class nnUNetTrainerV2_DDP(nnUNetTrainerV2):
         net.do_ds = ds
         return ret
 
-    def validate(self, do_mirroring: bool = True, use_sliding_window: bool = True, step_size: float = 0.5,
-                 save_softmax: bool = True, use_gaussian: bool = True, overwrite: bool = True,
+    def validate(self, do_mirroring: bool = True, use_sliding_window: bool = True,
+                 step_size: float = 0.5, save_softmax: bool = True, use_gaussian: bool = True, overwrite: bool = True,
                  validation_folder_name: str = 'validation_raw', debug: bool = False, all_in_gpu: bool = False,
-                 force_separate_z: bool = None, interpolation_order: int = 3, interpolation_order_z=0):
+                 segmentation_export_kwargs: dict = None):
         if self.local_rank == 0:
             if isinstance(self.network, DDP):
                 net = self.network.module
@@ -385,10 +351,10 @@ class nnUNetTrainerV2_DDP(nnUNetTrainerV2):
                 net = self.network
             ds = net.do_ds
             net.do_ds = False
-            ret = nnUNetTrainer.validate(self, do_mirroring, use_sliding_window, step_size, save_softmax, use_gaussian,
-                                         overwrite, validation_folder_name, debug, all_in_gpu,
-                                         force_separate_z=force_separate_z, interpolation_order=interpolation_order,
-                                         interpolation_order_z=interpolation_order_z)
+
+            ret = nnUNetTrainer.validate(self, do_mirroring, use_sliding_window, step_size, save_softmax,
+                                         use_gaussian, overwrite, validation_folder_name, debug, all_in_gpu,
+                                         segmentation_export_kwargs)
             net.do_ds = ds
             return ret
 
